#include <string.h>
#include <iostream>
#include <fstream>
#include <math.h>
using namespace std;

const int MAXT = 50;
const int MAXS = 100;
const int MAXP = 100;
const int MAXN = 80;

struct Tratto {
  float startx;
  float starty;
  float endx;
  float endy;
  float tempo;
};

struct Percorso {
  char nome[MAXS];
  Tratto tr[MAXT];
  int numTr;
};

struct ArchivioPercorsi {
  Percorso per[MAXP];
  int numPer;
};

int carica(ArchivioPercorsi* pa, const char nomef[]){
  fstream ff;
  ff.open(nomef, ios::in);
  if(!ff) return 0;
  while(true) {
    Percorso perc;
    if(! (ff >> perc.nome)) 
      break;
    int q;
    ff >> q;
    for(int i=0; i<q; i++) {
      Tratto t;
      ff >> t.startx;
      ff >> t.starty;
      ff >> t.endx;
      ff >> t.endy;
      ff >> t.tempo;
      perc.tr[i] = t;
    } 
    perc.numTr = q;
    pa->per[pa->numPer++] = perc;
  } 
  return pa->numPer;
}

float distanzaTratto(Tratto t){
  return sqrt((t.startx-t.endx)*(t.startx-t.endx) + (t.starty-t.endy)*(t.starty-t.endy));
}

float calcolaVel(Tratto t) {
  return distanzaTratto(t)/t.tempo;
}

float calcolaMediaVel(const Percorso* pp){
  float disttot = 0.0f;
  float tempotot = 0.0f;
  for(int i=0; i<pp->numTr; i++) {
    disttot += distanzaTratto(pp->tr[i]);
    tempotot += pp->tr[i].tempo;
  }
  return disttot/tempotot;
}

void disegnaGrafico(const Percorso* pp){
  float vmedia = calcolaMediaVel(pp);
  cout << pp->nome << endl;
  for(int i=0; i<pp->numTr; i++)
    if(calcolaVel(pp->tr[i]) > vmedia) cout << '*';
    else cout << ' ';
  cout << endl;
  for(int i=0; i<pp->numTr; i++)
    cout << '*';
  cout << endl;
}


float distanzaPercorso(const Percorso* pp) {
  float tot = 0.0f;
  for(int i=0; i<pp->numTr; i++)
    tot += distanzaTratto(pp->tr[i]);
  return tot;
}

float distanzaTotale(const ArchivioPercorsi* pa, const char n[]){
  float tot = 0.0f;
  for(int i=0; i<pa->numPer; i++)
    if(strcmp(pa->per[i].nome, n) == 0)
      tot += distanzaPercorso(&pa->per[i]);
  return tot;
}

struct Aux {
  char n[MAXN];
  int all;
};

void ordina(Aux v[], int n){
  for(int i=0; i<n-1; i++){
    int index = i;
    for(int j=i+1; j<n; j++)
      if(v[index].all<v[j].all)
        index = j;
    Aux a = v[i];
    v[i] = v[index];
    v[index] = a;
  }
}

void stampaNomiOrd(const ArchivioPercorsi* pa) {
  Aux aux[MAXP];
  int conta = 0;
  for(int i=0; i<pa->numPer; i++) {
    bool trovato = false;
    for(int j=0; j<conta; j++)
      if(strcmp(aux[j].n, pa->per[i].nome)==0) {
        aux[j].all++; 
        trovato = true;
      }
    if(!trovato){
      strcpy(aux[conta].n, pa->per[i].nome);
      aux[conta].all = 1;
      ++conta;
    }
  }
  ordina(aux, conta);
  for(int i=0; i<conta; i++)
    cout << aux[i].n << '\t' << aux[i].all << endl;
}

bool interno(Tratto t, float ax, float ay, float bx, float by) {
  return t.startx >= ax && t.startx <= bx && t.endx >= ax && t.endx <= bx &&   
         t.starty >= ay && t.starty <= by && t.endy >= ay && t.endy <= by;  
}

bool interno(const Percorso* pp, float ax, float ay, float bx, float by) {
  for(int i=0; i<pp->numTr; i++)
    if(!interno(pp->tr[i], ax, ay, bx, by))
      return false;
  return true;
}

int seleziona(const ArchivioPercorsi* pa, float ax, float ay, float bx, float by, Percorso v[]){
  int conta = 0;
  for(int i=0; i<pa->numPer; i++) {
    if(interno(&pa->per[i], ax, ay, bx, by))
      v[conta++] = pa->per[i];
  }
  return conta;
}

